#include <VRS51L3074_SDCC.h>

// 2's power, >= 256
#define BUFFERLEN 256
// mask for circular buffer,MSB only (256-> 1, 512 -> 3, 1024 -> 7 ...)
// as the buffer holds ints, size is 2*BUFFERLEN, so we mask out unused bits in MSB
#define BUFMASK 1

// Samples per ms
#define SPMS 39

unsigned char Flangeron=1;


void initAD(void);
void initPWM(void);
void PWMTMRInterrupt(void) interrupt 13 ;

/*
   delay: delay in ms
   depth: modulation depth (width) in ms
   wet:   processed signal level, 0-100%
   dry:   unprocessed signal level, 0-100%
   fback: feedback level, 0-100%
   step:  modulation rate, in Hzx10 (1=0.1Hz)
*/
void setupFlanger(int delay,int depth,unsigned char wet,unsigned char dry,unsigned char fback,unsigned char step);

void main (void) {

	PERIPHEN2 |= (1<<5);				//Enable the Arithmetic Unit
    initAD();
    initPWM();
        P2&=~0x1F;		// Make sure Outputs are Low
        P2PINCFG&=~0x1F;        // 11100000 P2.0,1,2,3,4 = output
	setupFlanger(0,6,70,70,20,1);
//	setupFlanger(0,6,0,70,0,1);
//	setupFlanger(0,6,70,0,0,1);
//	setupFlanger(0,6,70,70,0,1);
    GENINTEN = 0x03; //Enable global interrupt

    while (1){
    }
}

unsigned int fp,ep;               // pointers
unsigned int max_sweep,min_sweep; // sweep limits (offsets inside the buffer)
unsigned int sweep_h,sweep_l,sweep_step;   // sweep: cumulative sweep (32 bits in 16.16 and step in 0.16 
int wet_coef,dry_coef,fbk_coef;    // for summation, 1.15
unsigned char sweep_dir=0;         // sweep direction

__xdata at 0x0000 static int databuf[BUFFERLEN];

/* signed coeffs are not supported in setupFlanger for clarity,
   but they could be supported with minimal effort.
   The core routine uses signed coefficients which can add
   interesting phase reversal effects */
void setupFlanger(int delay,int depth,unsigned char wet,unsigned char dry,unsigned char fback,unsigned char step)
{
  sweep_step=13*step*depth; // step per sample in 0.16: 2*depth*step/samp = 65536*2*depth*(step/10)/(1000*SPMS)
  delay*=SPMS;            // delay in samples
  depth*=SPMS;            // depth in samples
  max_sweep=(BUFFERLEN-2)-delay;
  min_sweep=max_sweep-depth;
  sweep_h=min_sweep;
  sweep_l=0;
  wet_coef=327*wet;        // 1.15: 32768*wet%/100 (32768 not allowed)
  dry_coef=327*dry;
  fbk_coef=327*fback;
  fp=0;                 // initialize pointers
  ep=(min_sweep<<1);
}

/* 
   This function destroys A, B, R2, R3, R4, R5; be sure to save registers (if needed) before calling
   cursamp: current sample data in 1.15 format
   result: current output sample data in 1.15 format
*/

int Flanger(int cursamp) __naked
{
cursamp=cursamp;
_asm

  orl _DEVMEMCFG,#0x01		; Switch to SFR Page 1
  mov _AUCONFIG1,#0x08		; CAPREV  = 0  Previous Res capture is automatic
				; CAPMODE = 0  Capture of previous Result
				;		occurs when AUA0 is written into
				; OVCAPEN = 0  Capture on OV32 dis disabled
				; READCAP = 0  AURES contains current operation result
				; ADDSRC  = 10 Add SRC = AUPREV
				; MULCMD  = 00 Mul cmd = AUA x AUB

  mov _AUCONFIG2,#0xA0		; Clear the Arithmetic Unit registers
  mov _AUSHIFTCFG,#0x00         ; Disable barrel shifter

  mov r2,dpl            ; save current sample
  mov r3,dph
  mov dpl,_ep           ; point to current delayed sample
  mov dph,_ep+1
  movx a,@dptr          ; get it
  mov _AUA0,a           ; interpolate out=next*frac+cur*(1-frac)
  inc dptr              ; buffer does not wrap around in bytes but words
  movx a,@dptr
  mov _AUA1,a           ; AUA=cur
  clr C
  clr a
  subb a,_sweep_l       ; AUB=(1-frac) in 0.16, 1 is the 17th bit, frac=0 not allowed
  mov _AUB0,a
  clr a
  subb a,_sweep_l+1
  clr c                 ; move to 1.15, AU is signed and this operation is unsigned
  rrc a                 ; 16th bit will be 0
  mov _AUB1,a
  mov a,_AUB0
  rrc a
  mov _AUB0,a
  orl a,_AUB1           ; frac=0 ? (1-frac will be 0 if frac=0, in 0.16)
  jnz interp            ; No, keep working
  mov _AUB0,#1          ; Yes, skip interpolation.
                        ; (AUB1=0) AUB=1 in 16.0, so AURES=cur in 17.15
  sjmp nointerp
interp:
  inc dptr              ; next delayed sample
  anl dph,#BUFMASK      ; handle circular buffer
  movx a,@dptr
  mov _AUA0,a           ; capture cur*(1-frac) in 2.30
  inc dptr
  movx a,@dptr
  mov _AUA1,a           ; AUA=next
  mov a,_sweep_l+1      ; AUB=frac in 1.15
  clr c                 
  rrc a
  mov _AUB1,a
  mov a,_sweep_l        ; do next*frac in 2.30, accumulate
  rrc a
  mov _AUB0,a
  mov _AUSHIFTCFG,#0x31 ; Set the barrel shifter to perform a 15-bit right shift (convert 2.30 to 1.15)
nointerp:               ; warning, we come here with different setups of the barrel shifter
  mov r4,_AURES0        ; store out. If interpolated, out=AUPREV+AUA*AUB=next*frac+cur*(1-frac) in R4 R5 in 1.15
  mov r5,_AURES1        ; if skipping interpolation, out=cur in 17.15=1.15
  mov _AUA0,_fbk_coef   ; capture result in AUPREV
  mov _AUA1,_fbk_coef+1 ; load feedback coef in 1.15
  mov _AUCONFIG1,#0x02	; do AUA*AUPREV         CAPREV  = 0  Previous Res capture is automatic
						; CAPMODE = 0  Capture of previous Result
						;		occurs when AUA0 is written into
						; OVCAPEN = 0  Capture on OV32 dis disabled
						; READCAP = 0  AURES contains current operation result
						; ADDSRC  = 00 Add SRC = 0
						; MULCMD  = 10 Mul cmd = AUA x AUPREV
  mov _AUSHIFTCFG,#0x31 ; Set the barrel shifter to perform a 15-bit right shift (convert 2.30 to 1.15)
  mov dpl,_fp
  mov dph,_fp+1
  mov a,_AURES0         ; get out*feedback in 1.15
  add a,r2              ; add current sample
  movx @dptr,a          ; store in+fbk*buf in buffer
  mov a,_AURES1
  addc a,r3
  jnb PSW.2,fbk_ok      ; check for overflow
  mov b,a
  mov a,#0xFF            ; oops, need to saturate, check sign and saturate accordingly
  movx @dptr,a
  mov a,#0x7F
  jb b.7,fbk_ok
  clr a
  movx @dptr,a
  mov a,#0x80
fbk_ok:
  inc dptr
  movx @dptr,a          ; store MSB in buffer
                        ; save dptr for later
  mov _AUCONFIG1,#0x08				; CAPREV  = 0  Previous Res capture is automatic
						; CAPMODE = 0  Capture of previous Result
						;		occurs when AUA0 is written into
						; OVCAPEN = 0  Capture on OV32 dis disabled
						; READCAP = 0  AURES contains current operation result
						; ADDSRC  = 10 Add SRC = AUPREV
						; MULCMD  = 00 Mul cmd = AUA x AUB

  mov _AUCONFIG2,#0xA0		; Clear the Arithmetic Unit registers
  mov _AUSHIFTCFG,#0x00         ; Disable barrel shifter
  mov _AUA0,r2
  mov _AUA1,r3
  mov _AUB0,_dry_coef
  mov _AUB1,_dry_coef+1        ; plain input sample * dry level, 2.30
  mov _AUA0,r4
  mov _AUA1,r5
  mov _AUB0,_wet_coef
  mov _AUB1,_wet_coef+1        ; + processed sample * wet level, 2.30

  mov _AUSHIFTCFG,#0x01        ; Set the barrel shifter to perform a 1-bit left shift                                                        ; to get a 1.31 result
                               ; we will get a 1.15 result in dp later
  mov a,_sweep_dir
  jnz godown
  mov a,_sweep_l               ; sweep up
  add a,_sweep_step
  mov _sweep_l,a
  mov a,_sweep_l+1
  addc a,_sweep_step+1
  mov _sweep_l+1,a
  mov a,_sweep_h
  addc a,#0
  mov _sweep_h,a
  mov a,_sweep_h+1
  addc a,#0
  mov _sweep_h+1,a
  sjmp incptrs
  
godown:
  clr c
  mov a,_sweep_l               ; sweep down
  subb a,_sweep_step
  mov _sweep_l,a
  mov a,_sweep_l+1
  subb a,_sweep_step+1
  mov _sweep_l+1,a
  mov a,_sweep_h
  subb a,#0
  mov _sweep_h,a
  mov a,_sweep_h+1
  subb a,#0
  mov _sweep_h+1,a
incptrs:
  mov a,_fp
  add a,_sweep_h               ; ep = (int *) fp + sweep_h
  mov _ep,a
  mov a,_fp+1
  addc a,_sweep_h+1
  mov _ep+1,a
  mov a,_ep
  add a,_sweep_h               ; twice, buffer is for 16-bit data
  mov _ep,a
  mov a,_ep+1
  addc a,_sweep_h+1
  anl a,#BUFMASK               ; keep circular buffer
  mov _ep+1,a
  inc dptr                     ; fp++
  mov _fp,dpl
  anl dph,#BUFMASK
  mov _fp+1,dph

  clr c
  mov a,_sweep_h               ; sweep >= max_sweep ?
  subb a,_max_sweep
  mov a,_sweep_h+1
  subb a,_max_sweep+1
  jc chlow
  mov _sweep_dir,#1            ; yes, go down
  sjmp done
chlow:
  clr c
  mov a,_sweep_h               ; no, sweep < min_sweep
  subb a,_min_sweep
  mov a,_sweep_h+1
  subb a,_min_sweep+1
  jnc done
  mov _sweep_dir,#0            ; yes, go up
done:
  
  mov dph,_AURES3              ; get a 1.15 result in dptr, discarding the 16 LSBs
  mov dpl,_AURES2

  mov a,_AUCONFIG2
  jnb acc.0,out_ok             ; check for overflow
  mov dpl,#0xFF                ; oops, need to saturate, check sign and saturate accordingly
  mov dph,#0x7F
  mov a,_AURES3
  jb acc.7,out_ok
  mov dpl,#0
  mov dph,#0x80
out_ok:

  anl _DEVMEMCFG,#0xFE         ; Switch to SFR Page 0
  ret  
_endasm;
}

void ADInterrupt(void) interrupt 2 __naked using 1
{

_asm
  setb P2.0
        push psw
        push dpl
        push dph
        push ACC
        push b
        mov psw,#0x8                 ; select bank 1
	// Read SPI data, process it and put it in PWMMID
        mov dph,_SPIRXTX1
        mov dpl,_SPIRXTX0
        anl dph,#0x0F                ; dp = adcdata, 12-bit offset binary
        mov a,dph                    ; adcdata <<= 4
        swap a                       ; copied from sdcc
        anl a,#0xf0
        xch a,dpl
        swap a
        xch a,dpl
        xrl a,dpl
        xch a,dpl
        anl a,#0xf0
        xch a,dpl
        xrl a,dpl                    ; 16-bit offset binary
        xrl a,#0x80                  ; convert from offset binary to 2s-complement in 1.15
        mov dph,a
        
        mov A,_Flangeron
        jz flgoff
  setb P2.1
        lcall _Flanger
  clr P2.1
flgoff:
        mov A,dph                     ; discard LSBs, get only 8-bit data
        xrl A,#0x80                   ; convert to 8-bit offset binary
                                      ; if(adcdata==0)    // correct PWM flaw
	jnz no0
	inc A	                  ; adcdata++
	; Configure PWM5 MID value duty = adcdata
no0:	mov _PWMCFG,#0x05             ; Point to PWM5 MID LSB
	mov _PWMDATA,A                ; Set PWM MID LSB
        pop b
        pop ACC
        pop dph
        pop dpl
        pop psw
  clr P2.0
        reti
_endasm;
}


